#!/usr/bin/perl

######################################################################################################
# MP3 filename beautifier.
######################################################################################################
# 2003 by Henning Koch (jaz@netalive.org)
# Use on your own risk
# For private use only
# Please leave this notice intact when redistributing the program.
######################################################################################################

use MP3::Info;
use POSIX qw(locale_h);
use strict;

setlocale(LC_CTYPE, 'de_DE.ISO8859-1');


our %DONT_UC = (
	'but'	=> 1,
	'for'	=> 1,
	'is' => 1,
	'ft' => 1,
	'of' => 1,
	'to' => 1,
	'the' => 1,
	'my'	=> 1,
	'a' => 1,
	'as' => 1,
	'at' => 1,
	# 'not' => 1,
	'feat' => 1,
	'vs' => 1,
	'et' => 1,
	'in' => 1,
	'on' => 1,
	'auf' => 1,
	'wie' => 1,
	'die' => 1,
	'der' => 1,
	'das' => 1,
	'ist' => 1,
	'zu' => 1,
	'ein' => 1,
	'und' => 1,
	'da'	=> 1,
	'and' => 1,
	
	);
	
	
our @DOWNLOAD_SITES = (

	'MOD',
	'POW',
	'USF',
	'21stCentury',
	'TClub',
	'ESC',
	'RageMP3',
	'EtlMP3',
	'BMI',
	'UTE',
	
	);
	
	
our %FINAL_TOUCHES = (

	
	'T R A N C Ec O N T R O L' => 'Trance Control',
	'X - Perience' 		=> 'X-Perience',	# dont want padded dashes here
	'N - Trance' 		=> 'N-Trance',	# dont want padded dashes here
	'N - Joi' 		=> 'N-Joi',		# dont want padded dashes here
	'E - Motion' 		=> 'E-Motion',	# dont want padded dashes here
	'En - Motion' 		=> 'En-Motion',	# dont want padded dashes here
	'E - Type'		=> 'E-Type',
	'303Infinity'		=> '303infinity',
	'Atb'			=> 'ATB',
	'Nofx'			=> 'NOFX',
	'Fsol'			=> 'FSOL',
	'DJ Tiesto'		=> 'Tiesto, DJ',
	'Lyrix'			=> 'Lyrics',	# stupid kids
	'By Ahhhmp3s. com'	=> '',
	'12inch'		=> '12 Inch',
	'KHZ'			=> 'kHz',
	'ST.'			=> 'St.',
	'U. a.'			=> 'u.a.',
	'MP3Forge'		=> '',

	
	);

	
	
our $lc_letters = 'a-z' . join('', map(chr, 223 .. 255)); # Latin-1 characters
our $uc_letters = 'A-Z' . join('', map(chr, 192 .. 222)); # Latin-1 characters
our $letters = $lc_letters . $uc_letters;
our $lbound = '(?<![' . $letters . '\'])';
our $rbound = '(?![' . $letters . '\'])';



sub count_dashes {
	my ($count_me) = @_;
	my $dash_count = 0;
	while ($count_me =~ m/-/g) { $dash_count++ };
	return $dash_count;
}



sub count_padded_dashes {
	my ($count_me) = @_;
	my $dash_count = 0;
	while ($count_me =~ m/(?<=\s)-(?=\s)/g) { $dash_count++ };
	return $dash_count;
}


sub read_line_from_keyboard {
	my $key = <STDIN>;
	$key =~ s/[\r\n]//g;
	return $key;
}



sub beautify {

	my ($beautify_me) = @_;

	cleanup($beautify_me);


	if(count_dashes($beautify_me) > 1) {
		$beautify_me =~ s/(\s*-\s*|\s*\()(by )?([$uc_letters]{2,5})\s*$//;	# tags from download sites
		my $download_sites = join('|', map(quotemeta, @DOWNLOAD_SITES));
		$beautify_me =~ s/(\s*-\s*|\s*\()(by )?($download_sites)\s*$//i;	# tags from download sites
		}
		
	$beautify_me =~ s/(\s*\.?\s*mp3\s*\.?\s*)+$//gi;	# trailing .mp3

	$beautify_me =~ s/_{2,}/ - /g;		# multiple underscores to dashes
	
	$beautify_me =~ s/\.{3,}/ - /g;		# multiple dots to dashes

	$beautify_me =~ s/\~//g;		# tilde
	$beautify_me =~ s/!//g;			# exclamation marks
	$beautify_me =~ s/(\+)/ and /g;		# plus
	$beautify_me =~ s/_/ /g;		# underscores
	$beautify_me =~ s/\[/\(/g;		# square brackets
	$beautify_me =~ s/\]/\)/g;		# square brackets
	
	$beautify_me =~ s/\{([^\}]*)\}/ - $1 - /g;	# curly brackets
	
	$beautify_me =~ s/\b(www[\. ])?[$letters]+[\. ](com|net|co\s*\.\s*uk)\b//gi;	# URIs


	$beautify_me =~ s/\b(C\.J|D\.J|V\.J)\.?/substr($1,0,1) . substr($1,2,1)/gei;	# D.J. -> DJ

	$beautify_me =~ s/\btrack \d+\b//gi;	# remove "track 12"
	$beautify_me =~ s/\bdisc \d+\b//gi;	# remove "disc 2"
	$beautify_me =~ s/\bcopy of\b//gi;	# remove "copy of"
	$beautify_me =~ s/(copy)//gi;		# remove (copy)
	$beautify_me =~ s/\bkopie von\b//gi;	# remove "kopie von"
	$beautify_me =~ s/(kopie)//gi;		# remove (kopie)
	$beautify_me =~ s/\b\d+\.(.+\..+)$/$1/g;	# remove tracknumber "03.", but not a number preceding the file extension
	$beautify_me =~ s/\bCD[\s\d]+//gi;	# remove CD number
	$beautify_me =~ s/^[$letters]+\)//;		# remove CD number etc.
	
	cleanup($beautify_me);
	

	$beautify_me =~ s/^((Techno|Dance|Trance|Rock|Pop|Rave|Electronic|Punk|Classic|Disco|Jazz|Soundtrack|Christmas|Alternative|Weihnachten|\s|\d|s|\.|\(|\))+\s*(-|\))\s*)+//;	# remove leading genre titles
	$beautify_me =~ s/^ \(  (.+?)  \)   /$1 . ' - '/gex;	# artist name in brackets

	if($beautify_me !~ / /) {
		$beautify_me =~ s/([$uc_letters]|\d+)/" $1"/eg;
		}


	# $beautify_me =~ s/([$letters]) - \s+ ([^-]+?) \s - ([$letters])/$1 . ' - ' . $2 . ' - ' . $3/xgie;  # Dinge wie "blablabla- fewfewf -wgrwgr"
	
	
	
	$beautify_me =~ s/([$letters])-\s/$1 . ' - '/gei; # einseitig gepaddete dashes
	$beautify_me =~ s/\s-([$letters])/$1 . ' - '/gei; # einseitig gepaddete dashes


#	if($beautify_me =~ /FNT/i) {
#		print "$beautify_me\n";
#		exit;
#	}

	
	
	$beautify_me =~ s/([$letters']+)/if($DONT_UC{lc_latin($1)}) { lc_latin($1) } else { ucfirst_latin (lc_latin($1)) }/ge;	# uppercase words unless in DONT_UC



	$beautify_me = ucfirst_latin($beautify_me);				# first character always uppercase
	$beautify_me =~ s/(- |\()([$letters])/$1 . ucfirst_latin($2)/ge;	# uppercase characters after a dash
	$beautify_me =~ s/(\b|^)(CJ|DJ|VJ|VIP|NYC)(s?)\b/(uc $2) . (lc $3)/gei;	# uppercase common acronyms



	# print "$lbound\n$rbound\n";
	# exit;

	$beautify_me =~ s/$lbound([bcdfghjklmnpqrstvwxz]+)$rbound/uc $1/egi;	# uppercase words of consonants







	$beautify_me =~ s/\'(.)/"'" . lc_latin($1)/gsie;			# lowercase 'S that happened because of prior rule  "
	$beautify_me =~ s/\b(Mr|Ms|Hr|Fr|Mrs)\b/ucfirst_latin(lc_latin($1))/gsie;		# some consonant-only words I don't want all uppercase
	
	$beautify_me =~ s/\bVS\b/vs/gi;			# lowercase vs.
	$beautify_me =~ s/\bvs([^\.])/'vs.' . $1/gei;		# dot after vs
	
	# $beautify_me =~ s/$lbound(\d+)([$letters]{4,})/$1 $2/gi;
	


	$beautify_me =~ s/\b Dr\.+/Dr /igx;			# Dr. -> Dr
	$beautify_me =~ s/\b Dr\.* \b/Dr\./igx;		# Dr -> Dr.
	$beautify_me =~ s/\b(\d+)\.\s(\d+)\b/$1 . '.' . $2/ge; # Version 2.0

	$beautify_me =~ s/^The ([[$letters]\s]+?) -/$1 . ', The -'/gie;		# The Beatles -> Beatles, the
	$beautify_me =~ s/\b(CJ|DJ|VJ) ([$letters]+?) -/$2 . ', ' . $1 . ' -'/ge;	# DJ Tiesto -> Tiesto, DJ
	$beautify_me =~ s/, the\b/, The/g;					# uppercase the in band title

	$beautify_me =~ s/\binst\b/Instrumental/gi;

	$beautify_me =~ s/\bRMX\b/Remix/gi;			# RMX -> Remix

	$beautify_me =~ s/\b(feat|pres|ft)\b/feat\./gi;		# feat / pres -> feat.
	$beautify_me =~ s/feat\.+/feat\./gi;			# fix multiple .... caused by prior rule
	
	$beautify_me =~ s/Dont/Don't/g;			# correct spelling
	$beautify_me =~ s/Cant/Can't/g;			# correct spelling
	
	$beautify_me =~ s/-\ \d{1,2}\./- /g;
	
	# $beautify_me =~ s/U\. a\./\u.a\./g;
	

	$beautify_me =~ s/\b(x*?v*?i*?)\b/uc $1/gei; 		# roman numbers

	$beautify_me =~ s/\~\d+//g;				# filename~2
	$beautify_me =~ s/(\s*\(\s*\d*\s*\)\s*)+$/$2/ge;	# remove (2) from filename (2)


#	print "\n\n$beautify_me\n\n";
#	exit;
	cleanup($beautify_me);





	$beautify_me =~ s/$lbound(\d{1,2})\s*(PM|AM)$rbound/$1 . ' ' . uc($2)/xgei;

	if(count_dashes($beautify_me) <= 1) {
		$beautify_me =~ s/-/ - /g;		# pad dashes with spaces
	}


	$beautify_me =~ s/$lbound (\d+)(s|th|rd|nd|st) $rbound/$1 . lc($2)/gexi; # 80S->80s, 100TH -> 100th



	#if($beautify_me =~ /fatboy slim mix/i) {
	#	print "\n$beautify_me\n$beautify_me\n";
	#	exit;
	#}	
	
	foreach my $key (keys %FINAL_TOUCHES) {
	
		my $rekey = quotemeta $key;
		$beautify_me =~ s/$lbound $rekey $rbound/$FINAL_TOUCHES{$key}/igex;
	
	}




	cleanup($beautify_me);
	
	$beautify_me =~ s/\b(([$uc_letters]\.(\ |$)){2,})/my $x = $1; $x =~ s\/ \/\/g; $x . ' '/ge;  # F.S.O.L. soll so bleiben
	$beautify_me =~ s/\ +$//;

	return $beautify_me;

	


}



# system('cls');

print "\n\n";
print "*******************************************************************************\n";
print "* MP3 FILENAME BEAUTIFIER                                                     *\n";
print "* 2003 by henning koch (jaz\@netalive.org)                                     *\n";
print "* use on your own risk                                                        *\n";
print "* for private use only                                                        *\n";
print "* www.netalive.org                                                            *\n";
print "*******************************************************************************\n\n";



if($ARGV[0] eq '') {
	print 	"USAGE\n\n" .
		"   As parameters supply the filenames you wish to beautify.\n" .
		"\n" .
		
		"EXAMPLES\n\n" .
		"   perl filename-beautifier.pl d:/mp3/*.mp3\n" .
		"   perl filename-beautifier.pl d:/mp3/*.mp3 e:/movies/*.mpg\n" .
		"   perl filename-beautifier.pl d:/mp3/massive*.mp3\n".
		"   perl filename-beautifier.pl ./*.mp3\n\n";

	exit;
}



my @list = ();


print "Browsing your MP3 collection... ";

foreach my $param(@ARGV) {
	my($path, $filter) = split_path($param);
	if($path eq '') { $path = '.'; }
	
	if ($filter eq '') {
		print "Invalid parameter \"$param\"\n";
		exit; 
	}
		
	# print "\n$path || $filter\n";
	# exit;
	
	my @files = directory_listing($path, 0, 0, $filter);
	@list = (@list, map($_ = $path . '/' . $_, @files));	
}

print "done\n\n";


if(@list == ()) {
	print "No files found.\n";
	exit;
}
	



my $current = 0;
my $total = $#list + 1;

my $all_ok = 0;

foreach my $entry (lc_sort(@list)) {

	my ($path, $file) = split_path($entry);
	
	my $old_file = $file;
	$file =~ /^ (.*?)  \. ([^\.]+)$/x;
	
	my ($name_from_file, $ext) = ($1, $2);
	
	if($ext eq '') {
		print "No file extension found for \"$file\".\n";
		exit;
	}

	my ($tag);

	$tag = get_mp3tag($file, 1);
	my $id3v1_artist = $tag->{ARTIST};
	my $id3v1_title  = $tag->{TITLE};



	$tag = get_mp3tag($file, 2);
	my $id3v2_artist = $tag->{ARTIST};
	my $id3v2_title  = $tag->{TITLE};
	
	my $id3_artist = (length $id3v1_artist > length $id3v2_artist? $id3v1_artist : $id3v2_artist);
	my $id3_title  = (length $id3v1_title  > length $id3v2_title ? $id3v1_title  : $id3v2_title);
	
	my $name_from_id3 = $id3_artist . ' - ' . $id3_title;
	


	$name_from_id3 =~ s/[\\\/\:\?]/ - /g;  # machne zeichen darf es nicht als pfadname geben!
	#print "$file\n$name_from_file\n$name_from_id3\n";
	#exit;
	
	$name_from_id3  = beautify($name_from_id3);
	$name_from_file = beautify($name_from_file);
	
	
	my $source = 'File';
	# my $suggested_name = $name_from_file;

	
		if(	(count_dashes($name_from_file) == 0 || count_padded_dashes($name_from_file)  >  1 || length($name_from_file ) <  15) &&
			(count_dashes($name_from_id3) > 0 && count_padded_dashes($name_from_id3) == 1 && length($name_from_id3) >= 15)

		) { # kein dash oder zu kurz
			$source = 'ID3';
			# $suggested_name = $name_from_id3;
		}



	




	my $suggested_name = ($source eq 'File'? $name_from_file : $name_from_id3);

	$ext = lc_latin($ext);	

	$file = $suggested_name . '.' . $ext;
	
	# damit ich nicht zwei gleiche files ueberschreibe
	
	my $trail_no = 0;
	my $check = $file;
	
	unless (lc_latin($file) eq lc_latin($old_file))	{
	
		while(-f $path . '/' . $check) {
			$trail_no++;
			$check = $suggested_name . ' (' . $trail_no . ').' . $ext;
			# $check =~ s/(\.[$letters]+)$/' (' . $trail_no . ')' . $1/ie;
		}
	}
		
	$file = $check;




	my $test_old = $old_file;
	my $test_new = $file;
	$test_old =~ s/\s*\(\d+\)\s*//;	# wenn sie sich nur unterscheiden in (1) -> (3)
	$test_new =~ s/\s*\(\d+\)\s*//;	# dann weiter


	$current++;

	if($test_old eq $test_new) { # nur beim exakten match
		print "No suggestion for $file ...\n";
		next;
	}

	
	# system('cls');
	
	
#	print "+-------------------------------------------------------------------------+\n";
#	print "| MP3 Filename Beautifier v.2                                             |\n";
#	print "| Use on your own risk!                                                   |\n";
#	print "+-------------------------------------------------------------------------+\n";


	my $display = $old_file . '  [' . $current . '/' . $total . ']';

	# print "\n";
	print "+-----------------------------------------------------------------------------+\n";
	print "| " . pad_right($display, 75) . " |\n";
	print "+-----------------------------------------------------------------------------+\n\n";

	
#	print "File $current / $total   [Folder: $path]\n\n";
	
#	print "Rename : $old_file\n";
	print "  Suggestion: $file\n\n";
#	print "Source : $source\n\n";
	print "(y)es (n)o (a)ll (m)anual (q)uit: ";

	if ($all_ok) {
		print "a\n";
	}
	else {
	
		my $key = undef;
	
		while($key !~ /^[qmany]$/)
			{
			$key = read_line_from_keyboard();
			}
		# while (($key = &ReadKey(-1)) !~ /^[yanq]$/) {	}
		#
		#print "$key\n";


		if ($key eq 'q') {
			print "\nQuit.\n";
			exit;
		}

		if ($key eq 'a') {
			$all_ok = 1;
		}

		if ($key eq 'm') {
			print "\nEnter new filename: ";
			my $manual_file = read_line_from_keyboard();
			$file = $manual_file unless $manual_file eq '';
			
		}

		next if($key eq 'n');	
		
	}
		
	
	my $ok = rename($path . '/' . $old_file, $path . '/' . $file);
	
	print "\n";
	
	unless ($ok) {
		print "\nCould not rename last file.\n";
		exit;
	}
	
	
}
	
print "\nEnjoy the music!\n";

exit;





sub cleanup {

	# return;

	$_[0] =~ s/\.+$//g;		# trailing dots
	$_[0] =~ s/&/ & /g;		# pad ets with spaces
	$_[0] =~ s/\(/ \(/g;		# space before bracket
	$_[0] =~ s/^[\d\(\)\.]+\ //;	# remove leading track numbers
	$_[0] =~ s/([\.\,])/$1 /g;	# space after punctuation
	$_[0] =~ s/-\s*\(/\(/g;		# dash before bracket
	$_[0] =~ s/^ +//g;		# leading spaces
	$_[0] =~ s/^\s*-\s*//g;		# leading dashes
	$_[0] =~ s/[\`\]/\'/g;		# apostrophes that do not suck
	$_[0] =~ s/\(\s+/\(/g;		# unpad brackets inside
	$_[0] =~ s/\s+\)/\)/g;		# unpad brackets inside
	$_[0] =~ s/\s+\././g;		# remove spaces preceding fullstops
	$_[0] =~ s/\s*-\s*\././g;	# remove dashes preceding of fullstops
	$_[0] =~ s/-[\s\d]*-/-/g;	# superfluous dashes
	$_[0] =~ s/ +/ /g;		# multiple spaces
	$_[0] =~ s/\s*\(\)\s*//g;	# remove empty brackets
	$_[0] =~ s/[\s-]+$//ge;		# remove remaining leading bullshit
	$_[0] =~ s/^\d+ //;		# remove leading numbers
	

}





sub lc_latin {	# deutschen text

	my($txt) = @_;
	#$txt = lc $txt;
	$txt =~ s/([$uc_letters])/chr(ord($1) + 32)/ge;
	#$txt =~ s///gs;
	#$txt =~ s///gs;
	#$txt =~ s///gs;
	return $txt;

}


sub ucfirst_latin {	# deutschen text

	my($txt) = @_;
	$txt =~ s/^([$lc_letters])/chr(ord($1) - 32)/e;
	return $txt;

}



sub lc_latin {	# deutschen text

	my($txt) = @_;
	$txt = lc $txt;
	$txt =~ s///gs;
	$txt =~ s///gs;
	$txt =~ s///gs;
	return $txt;

}






sub lc_sort
	{
	return sort {lc($a) cmp lc($b)} @_;
	}

sub split_path {
	my($full_path) = @_;
	$full_path =~ /^((.*)[\/\\])?([^\/\\]*)$/s;
	return ($2, $3);
}
	
	
sub directory_listing	{

	my ($dir_path, $attrib, $recursive, $dos_match, $cut_extension) = @_;
	      ## $atrib: 0: nur files, 1: nur directories, 2: files & directories
	
	$dos_match =~ s/([^0-9a-z\*\+\?])/\\$1/gsi;
	   
	my %regRef = ('*','.*','?','.','+','.+');
	$dos_match =~ s/([\*\?\+])/ $regRef{$1} /ge;

	my @rev;
	
	# my mydir;
	
	opendir(mydir,$dir_path);
	
	foreach(readdir(mydir))
		{
		if ($_ !~ /^\.+$/ && ($dos_match eq '' || $dos_match eq '.*\..*' || $_ =~ /^ $dos_match $/xi  )   )

			{

			my $entry = $_;
			
			if (-f $dir_path . '/' . $entry && ($attrib == 0 || $attrib == 2)) {
				push(@rev, $entry);
				}


			if (-d $dir_path . '/' . $entry) {
				if ($attrib == 1 || $attrib == 2) {
					push(@rev, $entry);
					}
				
				if ($recursive)	{
					push(@rev, map($_ = $entry . '/' . $_,directory_listing($dir_path . '/' . $entry, $attrib, $recursive)));
					}
				}
			}
		}

	closedir(mydir);
	
	if ($cut_extension) {
		map($_ =~ s/\.[^\.]*$//, @rev);
		}
	
	return @rev;	
}




sub pad_right {
	my($s, $t) = @_;
	return $s . (' ' x ($t - length($s)));
}

sub pad_left {
	my($s, $t) = @_;
	return (' ' x ($t - length($s))) . $s;
}
